@@ -13,7 +13,7 @@ from contextlib import contextmanager
 from kuyruk import events, importer
 from kuyruk.queue import Queue
 from kuyruk.events import EventMixin
-from kuyruk.exceptions import Timeout
+from kuyruk.exceptions import Timeout, InvalidTask, ObjectNotFound, InvalidCall
 
 logger = logging.getLogger(__name__)
 
@@ -30,13 +30,81 @@ def profile(f):
     return inner
 
 
+def object_to_id(f):
+    """
+    If the Task is a class task, converts the first parameter to the id.
+
+    """
+    @wraps(f)
+    def inner(self, *args, **kwargs):
+        cls = self.cls or self.arg_class
+        if cls:
+            try:
+                obj = args[0]
+            except IndexError:
+                raise InvalidCall("You must give an instance of %s as first "
+                                  "argument." % cls)
+
+            if not isinstance(obj, cls):
+                raise InvalidCall("First argument must be an instance of %s." %
+                                  cls)
+
+            args = list(args)
+            try:
+                args[0] = args[0].id
+            except AttributeError:
+                raise InvalidCall("%s object must have an id attribute." % cls)
+        return f(self, *args, **kwargs)
+    return inner
+
+
+def id_to_object(f):
+    """
+    If the Task is a class task, converts the first argument to an object
+    by calling the get function of the class with the id.
+
+    """
+    @wraps(f)
+    def inner(self, *args, **kwargs):
+        cls = self.arg_class or self.cls
+        if cls:
+            if not args:
+                raise InvalidTask
+
+            obj_id = args[0]
+            obj = cls.get(obj_id)
+            if obj is None:
+                raise ObjectNotFound
+
+            if not isinstance(obj, cls):
+                raise ObjectNotFound("%s is not an instance of %s." %
+                                     (obj, cls))
+
+            args = list(args)
+            args[0] = obj
+
+        return f(self, *args, **kwargs)
+    return inner
+
+
+def send_client_signals(f):
+    """Sends the presend and postsend signals."""
+    @wraps(f)
+    def inner(self, *args, **kwargs):
+        self.send_signal(events.task_presend, args, kwargs, reverse=True)
+        rv = f(self, *args, **kwargs)
+        self.send_signal(events.task_postsend, args, kwargs)
+        return rv
+    return inner
+
+
 class Task(EventMixin):
 
     def __init__(self, f, kuyruk, queue='kuyruk', local=False, eager=False,
                  retry=0, max_run_time=None, arg_class=None):
         self.f = f
         self.kuyruk = kuyruk
-        self.queue = queue
+        self.queue_name = queue
         self.local = local
         self.eager = eager
         self.retry = retry
@@ -53,6 +121,8 @@ class Task(EventMixin):
     def __repr__(self):
         return "<Task of %r>" % self.name
 
+    @send_client_signals
+    @object_to_id
     def __call__(self, *args, **kwargs):
         """When a fucntion is wrapped with a task decorator it will be
         converted to a Task object. By overriding __call__ method we are
@@ -61,7 +131,6 @@ class Task(EventMixin):
 
         """
         logger.debug("Task.__call__ args=%r, kwargs=%r", args, kwargs)
-        self.send_signal(events.task_presend, args, kwargs, reverse=True)
 
         # These keyword argument allow the sender to override
         # the destination of the message.
@@ -70,21 +139,19 @@ class Task(EventMixin):
 
         if self.eager or self.kuyruk.config.EAGER:
             # Run the task in process
-            task_result = self.apply(*args, **kwargs)
+            task_result = self._run(*args, **kwargs)
         else:
             # Send it to the queue
             task_result = TaskResult(self)
             task_result.id = self.send_to_queue(
                 args, kwargs, host=host, local=local)
 
-        self.send_signal(events.task_postsend, args, kwargs)
-
         return task_result
 
     def __get__(self, obj, objtype):
         """If the task is accessed from an instance via attribute syntax
-        return a function for sending the task to queue, otherwise
-        return the task itself.
+        returns a bound task object that wraps the task itself, otherwise
+        returns the task itself.
 
         This is done for allowing a method to be converted to task without
         modifying the client code. When a function decorated inside a class
@@ -119,18 +186,8 @@ class Task(EventMixin):
 
         """
         logger.debug("Task.send_to_queueue args=%r, kwargs=%r", args, kwargs)
-        queue = self.queue
-        local_ = self.local
 
-        if local is not None:
-            local_ = local
-
-        if host:
-            queue = "%s.%s" % (self.queue, host)
-            local_ = False
-
-        with self.kuyruk.channel() as channel:
-            queue = Queue(queue, channel, local_)
+        with self.queue(host=host, local=local) as queue:
             desc = self.get_task_description(args, kwargs, queue.name)
             queue.send(desc)
 
@@ -139,14 +196,23 @@ class Task(EventMixin):
         # TODO no result backend is available yet
         return desc['id']
 
+    @contextmanager
+    def queue(self, host=None, local=None):
+        queue_ = self.queue_name
+        local_ = self.local
+
+        if local is not None:
+            local_ = local
+
+        if host:
+            queue_ = "%s.%s" % (self.queue_name, host)
+            local_ = False
+
+        with self.kuyruk.channel() as channel:
+            yield Queue(queue_, channel, local_)
+
     def get_task_description(self, args, kwargs, queue):
         """Return the dictionary to be sent to the queue."""
-
-        # For class tasks; replace the first argument with the id of the object
-        if self.cls or self.arg_class:
-            args = list(args)
-            args[0] = args[0].id
-
         return {
             'id': uuid1().hex,
             'queue': queue,
@@ -175,13 +241,20 @@ class Task(EventMixin):
         for sender in senders:
             signal.send(sender, task=self, args=args, kwargs=kwargs, **extra)
 
-    @profile
+    @send_client_signals
+    @object_to_id
     def apply(self, *args, **kwargs):
+        logger.debug("Task.apply args=%r, kwargs=%r", args, kwargs)
+        return self._run(*args, **kwargs)
+
+    @profile
+    @id_to_object
+    def _run(self, *args, **kwargs):
         """Run the wrapped function and event handlers."""
         def send_signal(signal, reverse=False, **extra):
             self.send_signal(signal, args, kwargs, reverse, **extra)
 
-        logger.debug("Task.apply args=%r, kwargs=%r", args, kwargs)
+        logger.debug("Task._apply args=%r, kwargs=%r", args, kwargs)
 
         result = TaskResult(self)
 
@@ -192,8 +265,7 @@ class Task(EventMixin):
         try:
             send_signal(events.task_prerun, reverse=True)
             with time_limit(limit):
-                # Call wrapped function
-                return_value = self.f(*args, **kwargs)
+                return_value = self.run(*args, **kwargs)
         except Exception:
             send_signal(events.task_failure, exc_info=sys.exc_info())
             raise
@@ -208,6 +280,14 @@ class Task(EventMixin):
         # task is sent to queue or executed in process with apply().
         return result
 
+    def run(self, *args, **kwargs):
+        """
+        Runs the wrapped function.
+        This method is intended to be overriden from subclasses.
+
+        """
+        return self.f(*args, **kwargs)
+
     @property
     def name(self):
         """Location for the wrapped function.
@@ -261,12 +341,8 @@ class BoundTask(Task):
     @wraps(Task.apply)
     def apply(self, *args, **kwargs):
         logger.debug("BoundTask.apply args=%r, kwargs=%r", args, kwargs)
-        # apply() may be called directly. Insert the bound object only if
-        # it is not inserted by __call__()
-        if args and args[0] is not self.obj:
-            args = list(args)
-            args.insert(0, self.obj)
-
+        args = list(args)
+        args.insert(0, self.obj)
         return super(BoundTask, self).apply(*args, **kwargs)
 
 
